<?php
/**
 * @file AGMO HTTP POST library
 *
 * Library implements basic interface for receiving and sending SMS.
 * Implements 3 classes
 *  - Agmo_Htpp_Post_Client - class implementing interface for sending MT SMS using PUSH method
 *  - Agmo_Htpp_Post_Server - class implementing interface for receiving MO SMS (and sending MT SMS in response)
 *  - Agmo_Htpp_Post_Exception - Exception object
 *
 * 
 * @author Jiri Bazant <jiri.bazant@comgate.cz>
 * @copyright Copyright (c) 2011, AGMO s.r.o.
 */

/**
 * @brief Catchable http post error
 * @author Jiri Bazant <jiri.bazant@comgate.cz>
 *
 */
class Agmo_Http_Post_Exception extends Exception {}


/**
 * @brief Http post client implementation
 *
 * @author Jiri Bazant <jiri.bazant@comgate.cz>
 *
 */
class Agmo_Http_Post_Client
{
    private $_url;
    private $_login;
    private $_pass;

    /**
     * @brief constructor. Set up connetction.
     *
     * @param string $url url of service, provided by AGMO
     * @param string $login login for http authentication, provided by AGMO
     * @param string $password password for http authentication, provided by AGMO
     * @throws Agmo_Http_Post_Exception on invalid arguments
     */
    public function __construct($url, $login, $password)
    {
        if (
          is_string($url) && !empty($url)
          && is_string($login) && !empty($login)
          && is_string($password) && !empty($password)
        ) {
            $this->_url = $url;
            $this->_login = $login;
            $this->_pass = $password;
        } else {
            throw new Agmo_Http_Post_Exception('Invalid settings', 1);
        }
    }

    /**
     * @brief URL getter
     */
    public function getUrl()
    {
        return $this->_url;
    }

    /**
     * @brief login getter
     */
    public function getLogin()
    {
        return $this->_login;
    }

    /**
     * @brief password getter
     */
    public function getPassword()
    {
        return $this->_pass;
    }

    /**
     * @brief get sms defaults
     */
    public function getSendSmsMTDefaults()
    {
        return array(
           'replyToId' => 0,
           'dataType' => 0,
        );
    }

    /**
     * @brief get wap push defaults
     */
    public function getWapPushDefaults()
    {
        return array(
           'replyToId' => 0,
        );
    }

    /**
     * @brief sends sms
     * @param array $params for more details see specification
     * @throws Agmo_Http_Post_Exception on invalid parameters specified
     */
    public function sendSmsMT($params)
    {
        if (is_array($params)) {
            //apply defaults
            $merged_params = array_merge($this->getSendSmsMTDefaults(), $params);

            //check params
            if ($this->_sendSmsMTValidate($params)) {
                //send request
                $merged_params['methodName'] = 'sendSmsMT';
                return $this->_curlSend($merged_params);

            } else {
                throw new Agmo_Http_Post_Exception('Invalid params specified', 3);
            }

        } else {
            throw new Agmo_Http_Post_Exception('$params must be an array', 2);
        }
    }

    /**
     * @brief Sends wap push
     * @param array $params for more details see specification
     * @throws Agmo_Http_Post_Exception on invalid parameters specified
     */
    public function sendWapPush($params)
    {
        if (is_array($params)) {
            //apply defaults
            $merged_params = array_merge($this->getWapPushDefaults(), $params);

            //check params
            if ($this->_sendWapPushValidate($params)) {
                //send request
                $merged_params['methodName'] = 'sendWapPush';
                return $this->_curlSend($merged_params);

            } else {
                throw new Agmo_Http_Post_Exception('Invalid params specified', 3);
            }

        } else {
            throw new Agmo_Http_Post_Exception('Params must be an array', 2);
        }
    }

    /**
     * @brief Sends subscription SMS
     * @throws Agmo_Http_Post_Exception on invalid parameters specified
     * @param array $params for more details see specification
     * @return mixed
     */
    public function sendSubscriptionSMS($params)
    {
        if (is_array($params)) {
            //check params
            if ($this->_sendSubscriptionSMSValidate($params)) {
                //send request
                $params['methodName'] = 'sendSubscriptionSMS';
                return $this->_curlSend($params);

            } else {
                throw new Agmo_Http_Post_Exception('Invalid params specified', 3);
            }

        } else {
            throw new Agmo_Http_Post_Exception('Params must be an array', 2);
        }
    }

    /**
     * @brief Sends personalized subscription SMS
     * @throws Agmo_Http_Post_Exception on invalid parameters specified
     * @param array $params for more details see specification
     * @return mixed
     */
    public function sendPersonalizedSubscriptionSMS($params)
    {
        if (is_array($params)) {
            //check params
            if ($this->_sendSubscriptionSMSValidate($params) && $this->_baseValidate($params)) {

                //send request
                $params['methodName'] = 'sendPersonalizedSubscriptionSMS';
                return $this->_curlSend($params);

            } else {
                throw new Agmo_Http_Post_Exception('Invalid params specified', 3);
            }

        } else {
            throw new Agmo_Http_Post_Exception('Params must be an array', 2);
        }
    }

    /**
     * @brief Send MMS
     * @throws Agmo_Http_Post_Exception
     * @param $params
     * @return
     */
    public function sendMmsMT($params)
    {
        if (is_array($params)) {
            //check params
            if ($this->_sendMmsMTValidate($params)) {
                //send request
                $params['methodName'] = 'sendMmsMT';
                return $this->_curlSend($params);
                
            } else {
                throw new Agmo_Http_Post_Exception('Invalid params specified', 3);
            }

        } else {
            throw new Agmo_Http_Post_Exception('Params must be an array', 2);
        }
    }

    /**
     * @brief sends post paramteres using cURL
     * @param array $params
     */
    protected function _curlSend($params)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $this->getUrl());
        curl_setopt($ch, CURLOPT_USERPWD, $this->getLogin() . ':' . $this->getPassword());
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));

        $res = curl_exec($ch);
        curl_close($ch);

        return $res;
    }

    /**
     * @brief Validate method for sendSmsMT and sendWapPush
     * @param array $params
     * @throws Agmo_Http_Post_Exception
     */
    protected function _baseValidate($params)
    {
        if ( //valide mobileNumber
            !array_key_exists('mobileNumber',$params)
            || !preg_match('/^[+][0-9]{11,12}$/', $params['mobileNumber'])
        ) {
            throw new Agmo_Http_Post_Exception('Invalid mobileNumber', 4);

        } elseif ( //check that optional replyToId is (not negative) integer, or not presented
            array_key_exists('replyToId', $params)
            && (!is_integer($params['replyToId'])
                || $params['replyToId'] < 0)
        ) {
            throw new Agmo_Http_Post_Exception('invalid replyToId specified', 6);

        } elseif (//check that optional datatype is in (0,1)
            array_key_exists('dataType', $params)
            && (!is_integer($params['dataType'])
                || 0 != $params['dataType'] && 1 != $params['dataType'])
        ) {
            throw new Agmo_Http_Post_Exception('invalid dataType specified', 7);

        } elseif (// check that optional serviceID is integer or omitted
            array_key_exists('serviceId', $params)
            && !is_null($params['serviceId'])
            && !is_integer($params['serviceId'])
        ) {
            throw new Agmo_Http_Post_Exception('serviceId is not type of integer', 8);
        }

        return true;
    }

    /**
     * @brief Validate method for sendSmsMT
     * @param array $params
     * @throws Agmo_Http_Post_Exception
     */
    protected function _sendSmsMTValidate($params)
    {
        if (//validate messageText
            !array_key_exists('messageText',$params)
            || empty($params['messageText'])
        ) {
            throw new Agmo_Http_Post_Exception('messageText not specified', 5);
        } else {
            return $this->_baseValidate($params);
        }
    }

    /**
     * @brief Validate method for sendWapPush
     * @param array $params
     * @throws Agmo_Http_Post_Exception
     */
    protected function _sendWapPushValidate($params)
    {
        if (//validate contentUrl
            !array_key_exists('contentUrl',$params)
            || empty($params['contentUrl'])
        ) {
            throw new Agmo_Http_Post_Exception('contentUrl not specified', 10);

        } elseif (//validate contentTitle
            !array_key_exists('contentTitle',$params)
            || empty($params['contentTitle'])
        ) {
            throw new Agmo_Http_Post_Exception('contentUrl not specified', 11);

        // validate length of sms
        } elseif (115 < strlen($params['contentTitle']) + strlen($params['contentUrl'])) {
            throw new Agmo_Http_Post_Exception('message too long', 14);
        } else {
            return $this->_baseValidate($params);
        }
    }

    /**
     * @brief Validate parameters for method sendSubSubscriptionSMS
     * @param array $params
     * @throws Agmo_Http_Post_Exception
     */
    protected function _sendSubscriptionSMSValidate($params)
    {
        //check content of subscription
        if (    (!array_key_exists('contentUrl',$params) || empty($params['contentUrl']))
            &&  (!array_key_exists('contentTitle',$params) || empty($params['contentTitle']))
            &&  (!array_key_exists('text',$params) || empty($params['text']))
        ) {
            throw new Agmo_Http_Post_Exception('Content of subscription not specified', 15);
        } elseif ((array_key_exists('contentUrl',$params) || !empty($params['contentUrl']))
            &&  (array_key_exists('text',$params) || !empty($params['text']))
        ) {
            throw new Agmo_Http_Post_Exception('Parameters ContentUrl and text cannot be used together', 17);
        } elseif (!array_key_exists('text',$params) || empty($params['text'])) {
            if (!array_key_exists('contentUrl',$params) || empty($params['contentUrl'])) {
                throw new Agmo_Http_Post_Exception('ContentUrl not specified', 16);
            }

        //check schedule
        } elseif (!array_key_exists('schedule',$params) || empty($params['schedule'])) {
            throw new Agmo_Http_Post_Exception('schedule not specified', 18);

        //check service
        } elseif (!array_key_exists('service',$params) || empty($params['service'])) {
            throw new Agmo_Http_Post_Exception('service not specified', 19);

        //check service
        } elseif (!array_key_exists('appNumber',$params) || empty($params['appNumber'])) {
            throw new Agmo_Http_Post_Exception('appNumber not specified', 20);
        }

        return true;
    }

    /**
     * @brief Validate paramters for method sendMmsMT
     * @throws Agmo_Http_Post_Exception
     * @param $params
     * @return bool
     */
    protected function _sendMmsMTValidate($params)
    {
        //check mobile numbers array (maybe we should check validity of numbers too)
        if (!array_key_exists('mobileNumbers', $params) || !is_array($params['mobileNumbers'])) {
            throw new Agmo_Http_Post_Exception('Parameter mobileNumbers not specified or not an array', 21);

        //check name array
        } elseif (!array_key_exists('name', $params) || !is_array($params['name'])) {
            throw new Agmo_Http_Post_Exception('Parameter name not specified or not an array', 22);

        //check mime array
        } elseif (!array_key_exists('mime', $params) || !is_array($params['mime'])) {
            throw new Agmo_Http_Post_Exception('Parameter mime not specified or not an array', 23);

        //check data array
        } elseif (!array_key_exists('data', $params) || !is_array($params['data'])) {
            throw new Agmo_Http_Post_Exception('Parameter data not specified or not an array', 24);

        //check subject
        } elseif (!array_key_exists('subject', $params) || empty($params['subject'])) {
            throw new Agmo_Http_Post_Exception('Parameter subject not specified or empty', 25);
        }

        return true;
    }
}


/**
 * @brief HTTP POST server implementation
 * @author Jiri Bazant <jiri.bazant@comgate.cz>
 *
 */
class Agmo_Http_Post_Server
{
    protected $_dlrHook;
    protected $_smsHook;

    protected $_login;
    protected $_pass;

    /**
     * @brief http server constructor
     * @param string $login accepted login
     * @param string $pass accepted password
     * @param string $smshook function callback optional
     * @param string $dlrhook function callback optional
     * @throws Agmo_Http_Post_Exception
     */
    public function __construct($login, $pass, $smshook = null, $dlrhook = null)
    {
        $this->_login = $login;
        $this->_pass = $pass;

        if ($smshook !== null) {
            if (function_exists($smshook)) {
                $this->_smsHook = $smshook;
            } else {
                throw new Agmo_Http_Post_Exception('$smshook must be null or valid function callback', 12);
            }
        }

        if ($dlrhook !== null) {
            if (function_exists($dlrhook)) {
                $this->_dlrHook = $dlrhook;
            } else {
                throw new Agmo_Http_Post_Exception('$dlrhook must be null or valid function callback', 13);
            }
        }
    }

    /**
     * @brief dispatch request
     *
     * Method dispatch request and returns response
     *
     * @throws Exception
     */
    public function dispatch()
    {
        $method = NULL;

        //Get method
        if (array_key_exists('methodName', $_POST)) {
            $method = $_POST['methodName'];
        }

        //authenticate
        if ($this->_authenticate()) {
            //if successfully authenticated try to dispatch request
            try {
                switch ($method) {
                    case 'receiveSmsMO':
                        $this->_receiveSmsMO();
                        break;
                    case 'receiveDlr':
                        $this->_receiveDlr();
                        break;
                    default:
                        header("HTTP/1.0 400 Bad Request");
                        header('Content-Type: text/plain');
                        printf("ERROR unknown method\r\n");
                }
            } catch (Agmo_Http_Post_Exception $e) {
                header("HTTP/1.0 400 Bad Request");
                header('Content-Type: text/plain');
                printf("ERROR " . $e->getMessage() ."\r\n");

                throw $e;

            } catch(Exception $e) {
                header("HTTP/1.0 500 Internal Error");
                header('Content-Type: text/plain');
                printf("ERROR " . $e->getMessage() ."\r\n");

                throw $e;
            }

        } else {
            header("HTTP/1.0 401 Unauthorized");
            header('Content-Type: text/plain');
            printf("ERROR Invalid credentials specified\r\n");
        }
    }

    protected function _authenticate()
    {
        if (isset($_SERVER['PHP_AUTH_USER'])
            && isset($_SERVER['PHP_AUTH_PW'])
            && $_SERVER['PHP_AUTH_USER'] == $this->_login
            && $_SERVER['PHP_AUTH_PW'] == $this->_pass
        ) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @brief dispetch receiveDlr request
     * Dispatch receiveDlr request. If dlrhook is specified, call it.
     */
    protected function _receiveDlr()
    {
        // extract parameters
        $names = array('msgRefId', 'mobileNumber', 'appNumber', 'msgTime', 'msgTextPart', 'dlrTime', 'status');
        $params = array();
        foreach ($names as $name) {
            if (array_key_exists($name, $_POST)) {
                $params[$name] = $_POST[$name];
            } else {
                $params[$name] = null;
            }
        }

        //do logic
        if ($this->_dlrHook) {
            call_user_func($this->_dlrHook, $params);
        }

        // return ok
        header("HTTP/1.0 200 OK");
        header('Content-Type: text/plain');

        printf("OK\r\n");
    }

    /**
     * @brief dispetch receiveSmsMO request
     * Dispatch receiveSmsMO request. If smshook is specified, call it.
     */
    protected function _receiveSmsMO()
    {
        // extract parameters
        $names = array('messageId', 'appNumber', 'mobileNumber', 'messageText',
                         'smscTime', 'billCode', 'operatorId', 'serviceId',
                         'refNo', 'maxNo', 'refId');

        $params = array();
        foreach ($names as $name) {
            if (array_key_exists($name, $_POST)) {
                $params[$name] = $_POST[$name];
            } else {
                $params[$name] = null;
            }
        }

        //array of optional responses
        $responses = array();

        //do logic
        if ($this->_smsHook) {
            $responses = call_user_func($this->_smsHook, $params);
        }

        // return ok and response
        header("HTTP/1.0 200 OK");
        header('Content-Type: text/plain');

        printf("OK\r\n");

        // add responses
        foreach ($responses as $response) {
           printf("%s\r\n", http_build_query($response));
        }
    }
}
